#include "Lights.h"

#include <stdexcept>

GLuint Light::count = 0; // количество используемых источников (должно быть <= MAX_LIGHTS)
LightData Light::data[MAX_LIGHTS]; // Массив данных по источникам света

// возвращает размер буфера в байтах
int Light::getUBOsize()
{
    return sizeof(LightData) * MAX_LIGHTS + sizeof(GLuint);
}

// Загрузка данных в буфер
void Light::upload(UBO& lights_data)
{
    GLuint LightDataSize = sizeof(LightData); // Одного экземпляра структуры LightData

    if (count)
        lights_data.loadSub(data, sizeof(LightData)*count); // Загрузка данных об источниках

    // Загружаем кол-во источников
    lights_data.loadSub(&count, sizeof(count), LightDataSize*MAX_LIGHTS);
}

// Метод пересчета матрицы трансформации по необходимости, должен сбрасывать флаг changed
void Light::recalcMatrices()
{
    // Если были изменения - необходимо загрузить данные
    if (changed || parent_changed)
        uploadReq = true;
    
    // Выполняем вычисление матриц методом родительского класса
    Node::recalcMatrices(); 
}

// Константный доступ к цвету
const glm::vec3& Light::c_color() const
{
    return color;
}

// Неконстантная ссылка для изменений цвета
glm::vec3& Light::e_color()
{
    uploadReq = true;

    return color;
}

// Проверка что не взаимодествуем с пустым источником
void Light::check_id()
{
    if (index < 0
    ||  index >= count)
        throw std::runtime_error("Попытка использовать ссылку на пустой или некорректный источник");
}

// Преобразует информацию об источнике в структуру LightData
void Light::toData()
{
    check_id(); // Проверка на работу с корректным индексом

    data[index].position = glm::vec3(result_transform[3]); // Позиция из матрицы трансформации
    data[index].color = color; // Цвет
}
